==> cmomy-0.15.0-py3-none-any.whl: cmomy/_lib/push.py <==
"""Vectorized pushers."""

from __future__ import annotations

from functools import partial
from typing import TYPE_CHECKING

import numba as nb

from . import _push
from .decorators import myguvectorize

if TYPE_CHECKING:
    from numpy.typing import NDArray

    from ..typing import FloatT, NDGeneric


_PARALLEL = False
_decorator = partial(myguvectorize, parallel=_PARALLEL)


@_decorator(
    "(),(),(mom)",
    [
        (nb.float32, nb.float32, nb.float32[:]),
        (nb.float64, nb.float64, nb.float64[:]),
    ],
)
def push_val(x: NDGeneric[FloatT], w: NDGeneric[FloatT], out: NDArray[FloatT]) -> None:
    _push.push_val(x, w, out)


@_decorator(
    "(sample),(sample),(mom)",
    [
        (nb.float32[:], nb.float32[:], nb.float32[:]),
        (nb.float64[:], nb.float64[:], nb.float64[:]),
    ],
)
def reduce_vals(x: NDArray[FloatT], w: NDArray[FloatT], out: NDArray[FloatT]) -> None:
    for i in range(x.shape[0]):
        _push.push_val(x[i], w[i], out)


# @_decorator(
#     "(sample),(sample),(mom)",
#     [
#         (nb.float32[:], nb.float32[:], nb.float32[:]),
#         (nb.float64[:], nb.float64[:], nb.float64[:]),
#     ],
# )
# def reduce_vals_multipass(
#     x: NDArray[FloatT], w: NDArray[FloatT], out: NDArray[FloatT]
# ) -> None:
#     # first calculate average
#     xave = 0.0
#     wsum = 0.0
#     for i in range(x.shape[0]):
#         ww = w[i]
#         xave += x[i] * ww
#         wsum += ww

#     xave /= wsum
#     out[...] = 0.0
#     out[0] = wsum
#     out[1] = xave

#     # sum other moments
#     nmom = out.shape[-1]
#     if nmom > 2:
#         for i in range(x.shape[0]):
#             xx = x[i]
#             ww = w[i]
#             for m in range(2, nmom):
#                 out[m] += ww * (xx - xave) ** m

#         for m in range(2, nmom):
#             out[m] /= wsum


@_decorator(
    "(),(vars),(),(mom)",
    [
        (nb.float32, nb.float32[:], nb.float32, nb.float32[:]),
        (nb.float64, nb.float64[:], nb.float64, nb.float64[:]),
    ],
)
def push_stat(
    a: NDGeneric[FloatT],
    v: NDArray[FloatT],
    w: NDGeneric[FloatT],
    out: NDArray[FloatT],
) -> None:
    _push.push_stat(a, v, w, out)


@_decorator(
    "(sample),(sample,vars),(sample),(mom)",
    [
        (nb.float32[:], nb.float32[:, :], nb.float32[:], nb.float32[:]),
        (nb.float64[:], nb.float64[:, :], nb.float64[:], nb.float64[:]),
    ],
)
def reduce_stats(
    a: NDArray[FloatT],
    v: NDArray[FloatT],
    w: NDArray[FloatT],
    out: NDArray[FloatT],
) -> None:
    for i in range(a.shape[0]):
        _push.push_stat(a[i], v[i, :], w[i], out)


@_decorator(
    "(mom),(mom)",
    signature=[
        (nb.float32[:], nb.float32[:]),
        (nb.float64[:], nb.float64[:]),
    ],
)
def push_data(data: NDArray[FloatT], out: NDArray[FloatT]) -> None:
    _push.push_data(data, out)


@_decorator(
    "(sample, mom),(mom)",
    [
        (nb.float32[:, :], nb.float32[:]),
        (nb.float64[:, :], nb.float64[:]),
    ],
)
def reduce_data(data: NDArray[FloatT], out: NDArray[FloatT]) -> None:
    for i in range(data.shape[0]):
        _push.push_data(data[i, :], out)


@_decorator(
    "(sample, mom) -> (mom)",
    [
        (nb.float32[:, :], nb.float32[:]),
        (nb.float64[:, :], nb.float64[:]),
    ],
    writable=None,
)
def reduce_data_fromzero(
    data: NDArray[FloatT],
    out: NDArray[FloatT],
) -> None:
    out[...] = 0.0
    for i in range(data.shape[0]):
        _push.push_data(data[i, :], out)

==> cmomy-0.20.0-py3-none-any.whl: cmomy/_lib/push.py <==
"""Vectorized pushers."""

from __future__ import annotations

from functools import partial
from typing import TYPE_CHECKING

import numba as nb

from . import _push
from .decorators import myguvectorize

if TYPE_CHECKING:
    from numpy.typing import NDArray

    from cmomy.core.typing import FloatT, NDGeneric


_PARALLEL = False
_decorator = partial(myguvectorize, parallel=_PARALLEL)


@_decorator(
    "(mom),(),()",
    [
        (nb.float32[:], nb.float32, nb.float32),
        (nb.float64[:], nb.float64, nb.float64),
    ],
)
def push_val(
    out: NDArray[FloatT],
    x: NDGeneric[FloatT],
    w: NDGeneric[FloatT],
) -> None:
    _push.push_val(x, w, out)


@_decorator(
    "(mom),(sample),(sample)",
    [
        (nb.float32[:], nb.float32[:], nb.float32[:]),
        (nb.float64[:], nb.float64[:], nb.float64[:]),
    ],
)
def reduce_vals(
    out: NDArray[FloatT],
    x: NDArray[FloatT],
    w: NDArray[FloatT],
) -> None:
    for i in range(x.shape[0]):
        _push.push_val(x[i], w[i], out)


@_decorator(
    "(mom),(mom)",
    signature=[
        (nb.float32[:], nb.float32[:]),
        (nb.float64[:], nb.float64[:]),
    ],
)
def push_data(data: NDArray[FloatT], out: NDArray[FloatT]) -> None:
    _push.push_data(data, out)


@_decorator(
    "(sample, mom),(mom)",
    [
        (nb.float32[:, :], nb.float32[:]),
        (nb.float64[:, :], nb.float64[:]),
    ],
)
def reduce_data(data: NDArray[FloatT], out: NDArray[FloatT]) -> None:
    for i in range(data.shape[0]):
        _push.push_data(data[i, :], out)


@_decorator(
    "(mom),(),(mom)",
    signature=[
        (nb.float32[:], nb.float32, nb.float32[:]),
        (nb.float64[:], nb.float64, nb.float64[:]),
    ],
)
def push_data_scale(data: NDArray[FloatT], scale: float, out: NDArray[FloatT]) -> None:
    _push.push_data_scale(data, scale, out)


@_decorator(
    "(sample, mom) -> (mom)",
    [
        (nb.float32[:, :], nb.float32[:]),
        (nb.float64[:, :], nb.float64[:]),
    ],
    writable=None,
)
def reduce_data_fromzero(
    data: NDArray[FloatT],
    out: NDArray[FloatT],
) -> None:
    out[...] = 0.0
    for i in range(data.shape[0]):
        _push.push_data(data[i, :], out)


@_decorator(
    "(sample, mom) -> (sample, mom)",
    [
        (nb.float32[:, :], nb.float32[:, :]),
        (nb.float64[:, :], nb.float64[:, :]),
    ],
    writable=None,
)
def cumulative(
    data: NDArray[FloatT],
    out: NDArray[FloatT],
) -> None:
    out[0, ...] = data[0, ...]
    for i in range(1, data.shape[0]):
        out[i, ...] = data[i, ...]
        _push.push_data(out[i - 1, ...], out[i, ...])


@_decorator(
    "(sample, mom) -> (sample, mom)",
    [
        (nb.float32[:, :], nb.float32[:, :]),
        (nb.float64[:, :], nb.float64[:, :]),
    ],
    writable=None,
)
def cumulative_inverse(
    data: NDArray[FloatT],
    out: NDArray[FloatT],
) -> None:
    out[0, ...] = data[0, ...]
    for i in range(1, data.shape[0]):
        out[i, ...] = data[i, ...]
        _push.push_data_scale(data[i - 1, ...], -1.0, out[i, ...])
